/* ************ COPYRIGHT 2011, H. G. Katzgraber. ********************* */
/*                                                                      */
/* EX 2:        Estimate Pi with simple Monte Carlo                     */
/*                                                                      */
/*              Computes up to 10^N log-space random numbers and prints */
/*              estimates for Pi (and deviation) every 10^k steps.      */
/*		The estimates are averaged over M = 10 runs to improve	*/
/*		statistics. Feel free to change the numbers.		*/
/* INSTR:       Compile with Makefile via 'make', execute with ./runme  */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "r1279.h"
#include "nrutil.h"		/* helper routines for array allocation	*/

#define N	7           	/* number of 10^N random nums    	*/
#define M	10	    	/* number of runs to improve statistics */

long seedgen(void);

int main()
{
    int 	n, k, m, i;
    double 	x, y, estimate, stat_estimate, pi;
    int		count;
    long 	seed;
    
    seed        = seedgen();    /* have seedgen compute a random seed   */
    setr1279(seed);             /* seed the genertor                    */

    pi 		= 4.0*atan(1.0);/* use the arctan function to get pi	*/

    printf("MCS     Pi est   Error\n");

    for(k = 1; k <= N; k++){
	m = pow(10,k);
	stat_estimate = 0.0;
	for(i = 1; i <= M; i++){
	    count    = 0.0;
	    for(n = 1; n <= m; n++){
		x = r1279();
		y = r1279();
		if(x*x + y*y < 1){
		    count++;
		}
	    }
	    estimate = 4.0*((double) count)/((double) m);
	    stat_estimate += estimate;
	}
	stat_estimate /= ((double) M);
	printf("%2.1e %f %6.5e\n",
		((double) m),
		stat_estimate,
		fabs(stat_estimate - pi)
		);
    }
    
    return EXIT_SUCCESS;
}
