/* ************ COPYRIGHT 2011, H. G. Katzgraber. ********************* */
/*                                                                      */
/* EX 5:        Estimate Pi with Markov-chain Monte Carlo               */
/*                                                                      */
/*              Computes up to 10^N log-space random numbers and prints */
/*              estimates for Pi (and deviation) every 10^k steps.      */
/*              The estimates are averaged over M = 10 runs to improve  */
/*              statistics. Feel free to change the numbers.            */
/*		The program also prints the acceptance probability for	*/
/*		the Monte Carlo moves. Tune the parameter P to see how	*/
/*		convergence changes.					*/
/*		NOTE: this program ignores autocorrelation effects!!!	*/
/* INSTR:       Compile with Makefile via 'make', execute with ./runme  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "r1279.h"
#include "nrutil.h"		/* helper routines for array allocation	*/

#define N	7           	/* number of 10^N random nums    	*/
#define M	10	    	/* number of runs to improve statistics */
#define	P	0.50		/* maximal random walk step           	*/
				/* for P = 1.00 we get ~ 50% acceptance	*/

long 	seedgen(void);
double	range_rand(double x, double min, double max);

int main()
{
    int 	n, k, m, i;
    double 	x, y, dx, dy, estimate, stat_estimate, ar, pi;
    int		count, acceptance_count;
    long 	seed;
    
    seed        = seedgen();    /* have seedgen compute a random seed   */
    setr1279(seed);             /* seed the genertor                    */

    pi 		= 4.0*atan(1.0);/* use the arctan function to get pi	*/

    printf("MCS     Pi est   Error       Acceptance rate\n");

    for(k = 1; k <= N; k++){
	m = pow(10,k);
	stat_estimate 	 = 0.0;
	acceptance_count = 0.0;
	for(i = 1; i <= M; i++){
	    count    = 0.0;
	    x	     = 0.25;	/* select a random spot for a start */
	    y        = 0.25;
	    for(n = 1; n <= m; n++){
		dx = range_rand(r1279(),-P,P);
		dy = range_rand(r1279(),-P,P);
		if((fabs(x+dx) < 1) && (fabs(y+dy) < 1)){
		    x = x + dx;
		    y = y + dy;
		    acceptance_count++;
		}
		if(x*x + y*y < 1){
		    count++;
		}
	    }
	    estimate = 4.0*((double) count)/((double) m);
	    stat_estimate += estimate;
	}
	stat_estimate /= ((double) M);
	ar = ((double) acceptance_count)/(((double) M)*((double) m));
	printf("%2.1e %f %6.5e %6.5e\n",
		((double) m),
		stat_estimate,
		fabs(stat_estimate - pi),
		ar
		);
    }
    
    return EXIT_SUCCESS;
}

double range_rand(double x, double min, double max){

    double 	y;

    y = min + (max - min)*x;

    return y;
}
